@@ -20,7 +20,7 @@
 # SOFTWARE.
 #
 
-from ..compat import long_
+from ..compat import basestring_, long_, unicode_
 
 import datetime
 import decimal
@@ -30,11 +30,12 @@ import decimal
 class PyodbcMockFactory(object):
 
     # executeCallback(resourceString, query, *args), returns [(columnNames, columnDatas), ...]
-    def __init__(self, executeCallback = None):
+    def __init__(self, executeCallback = None, searchescape = '\\'):
         self.executeCallback = executeCallback or SimpleExecuteCallback()
+        self._searchescape = searchescape
 
     def open(self, resourceString):
-        return PyodbcConnectionMock(resourceString, self.executeCallback)
+        return PyodbcConnectionMock(resourceString, self.executeCallback, self._searchescape)
 
     def close(self, resource):
         resource.close()
@@ -70,47 +71,88 @@ class PyodbcConnectionMock(object):
 
     pyodbc = PyodbcMock
 
-    def __init__(self, resourceString, executeCallback):
+    def __init__(self, resourceString, executeCallback, searchescape):
+
+        # Parse resource string for properties
+        properties = dict((k.lower(), v) for k, v in (x.split('=') for x in resourceString.split(';') if x))
 
-        self.resourceString = resourceString
-        self.properties = dict((k.lower(), v) for k, v in (x.split('=') for x in resourceString.split(';') if x))
         self.executeCallback = executeCallback
-        self.isClosed = False
-        self.cursors = []
+        self._searchescape = searchescape
+        self._autocommit = (properties.get('autocommit', 'false') == 'true')
+        self._readonly = (properties.get('readonly', 'false') == 'true')
+        self._timeout = int(properties.get('timeout', '0'))
+        self._unicode_results = (properties.get('unicode_results', 'false') == 'true')
+        self._isClosed = False
+        self._cursors = []
 
     @property
     def autocommit(self):
-        return self.properties.get('autocommit', 'false').strip().lower() == 'true'
+        return self._autocommit
+
+    @autocommit.setter
+    def autocommit(self, value):
+
+        if self._isClosed:
+            raise PyodbcMock.ProgrammingError('Attempt to set autocommit on an already-closed connection')
+
+        if not isinstance(value, bool):
+            raise PyodbcMock.ProgrammingError('Attempt to set autocommit to a non-bool value')
+
+        self._autocommit = value
 
     def close(self):
 
-        if self.isClosed:
+        if self._isClosed:
             raise PyodbcMock.ProgrammingError('Attempt to close an already-closed connection')
 
-        for cursor in self.cursors:
-            if not cursor.isClosed:
+        for cursor in self._cursors:
+            if not cursor._isClosed:
                 raise PyodbcMock.ProgrammingError('Attempt to close a connection with open cursors')
 
-        self.isClosed = True
+        self._isClosed = True
 
     def cursor(self):
 
-        if self.isClosed:
+        if self._isClosed:
             raise PyodbcMock.ProgrammingError('Attempt to get a cursor on an already-closed connection')
 
         cursor = PyodbcCursorMock(self)
-        self.cursors.append(cursor)
+        self._cursors.append(cursor)
         return PyodbcCursorContext(cursor)
 
     def execute(self, query, *args):
 
-        if self.isClosed:
+        if self._isClosed:
             raise PyodbcMock.ProgrammingError('Attempt to execute with an already-closed connection')
 
         cursor = PyodbcCursorMock(self).execute(query, *args)
-        self.cursors.append(cursor)
+        self._cursors.append(cursor)
         return PyodbcCursorContext(cursor)
 
+    def rollback(self):
+
+        if self._isClosed:
+            raise PyodbcMock.ProgrammingError('Attempt to rollback an already-closed connection')
+
+    @property
+    def searchescape(self):
+        return self._searchescape
+
+    @property
+    def timeout(self):
+        return self._timeout
+
+    @timeout.setter
+    def timeout(self, value):
+
+        if self._isClosed:
+            raise PyodbcMock.ProgrammingError('Attempt to set timeout on an already-closed connection')
+
+        if not isinstance(value, (int, long_)) or isinstance(value, bool):
+            raise PyodbcMock.ProgrammingError('Attempt to set timeout to a non-integer value')
+
+        self._timeout = value
+
 
 # pyodbc row mock
 class PyodbcRowMock(object):
@@ -142,7 +184,7 @@ class PyodbcRowMock(object):
 
 
 # Assert rowSets is properly formed - [(columnNames, columnDatas), ...]
-def assertRowSets(rowSets):
+def assertRowSets(rowSets, connection = None):
 
     assert isinstance(rowSets, (tuple, list)), \
         'Invalid rowset collection %r - list expected' % (rowSets,)
@@ -164,18 +206,24 @@ def assertRowSets(rowSets):
             assert len(rowSet[0]) == len(columnData), \
                 'Column data tuple has different length than column names tuple (%r, %r)' % (rowSet[0], columnData)
             for data in columnData:
-                assert isinstance(data, (type(None),
-                                         str,
-                                         bytearray,
-                                         bool,
-                                         datetime.date,
-                                         datetime.time,
-                                         datetime.datetime,
-                                         int,
-                                         long_,
-                                         float,
-                                         decimal.Decimal)), \
-                                         'Invalid column data %r' % (data,)
+                if connection is not None and isinstance(data, basestring_):
+                    if connection._unicode_results:
+                        assert isinstance(data, unicode_), 'unicode_results connection result set strings must be unicode'
+                    else:
+                        assert isinstance(data, str), 'non-unicode_results connection result set strings must be non-unicode'
+                else:
+                    assert isinstance(data, (type(None),
+                                             basestring_,
+                                             bytearray,
+                                             bool,
+                                             datetime.date,
+                                             datetime.time,
+                                             datetime.datetime,
+                                             int,
+                                             long_,
+                                             float,
+                                             decimal.Decimal)), \
+                                             'Invalid column data %r' % (data,)
 
 
 # pyodbc cursor mock
@@ -188,58 +236,58 @@ class PyodbcCursorMock(object):
 
     def _reset(self):
 
-        self.isClosed = False
-        self.isCommit = False
-        self.rowSets = None
-        self.ixRowSet = 0
-        self.ixRow = 0
+        self._isClosed = False
+        self._isCommit = False
+        self._rowSets = None
+        self._ixRowSet = 0
+        self._ixRow = 0
 
     def close(self):
 
-        if self.isClosed or self.connection.isClosed:
+        if self._isClosed or self.connection._isClosed:
             raise PyodbcMock.ProgrammingError('Attempt to close an already-closed cursor')
 
-        self.isClosed = True
+        self._isClosed = True
 
     def commit(self):
 
         if self.connection.autocommit:
             raise PyodbcMock.ProgrammingError('Attempt to commit an autocommit cursor')
-        if self.rowSets is None:
+        if self._rowSets is None:
             raise PyodbcMock.ProgrammingError('Attempt to commit cursor before execute')
-        if self.isClosed or self.connection.isClosed:
+        if self._isClosed or self.connection._isClosed:
             raise PyodbcMock.ProgrammingError('Attempt to commit an already-closed cursor')
-        if self.isCommit:
+        if self._isCommit:
             raise PyodbcMock.ProgrammingError('Attempt to commit an already-committed cursor')
 
-        self.isCommit = True
+        self._isCommit = True
 
     def execute(self, query, *args):
 
         assert isinstance(query, str)
 
-        if self.rowSets is not None:
+        if self._rowSets is not None:
             raise PyodbcMock.ProgrammingError('Attempt to execute on an already-executed-on cursor')
-        if self.isClosed or self.connection.isClosed:
+        if self._isClosed or self.connection._isClosed:
             raise PyodbcMock.ProgrammingError('Attempt to execute on a closed cursor')
 
         # Call the execute callback
-        self.rowSets = self.connection.executeCallback(query, *args)
-        assertRowSets(self.rowSets)
+        self._rowSets = self.connection.executeCallback(query, *args)
+        assertRowSets(self._rowSets, connection = self.connection)
 
         return self
 
     def nextset(self):
 
-        if self.isClosed or self.connection.isClosed:
+        if self._isClosed or self.connection._isClosed:
             raise PyodbcMock.ProgrammingError('Attempt to call nextset on a closed cursor')
-        if self.isCommit:
+        if self._isCommit:
             raise PyodbcMock.ProgrammingError('Attempt to call nextset on a committed cursor')
-        if self.rowSets is None or self.ixRowSet + 1 >= len(self.rowSets):
+        if self._rowSets is None or self._ixRowSet + 1 >= len(self._rowSets):
             raise PyodbcMock.ProgrammingError('Attempt to call nextset when no more row sets available')
 
-        self.ixRowSet += 1
-        self.ixRow = 0
+        self._ixRowSet += 1
+        self._ixRow = 0
 
     def fetchone(self):
         try:
@@ -252,19 +300,19 @@ class PyodbcCursorMock(object):
 
     def __next__(self):
 
-        if self.isClosed or self.connection.isClosed:
+        if self._isClosed or self.connection._isClosed:
             raise PyodbcMock.ProgrammingError('Attempt to iterate a closed cursor')
-        if self.isCommit:
+        if self._isCommit:
             raise PyodbcMock.ProgrammingError('Attempt to iterate a committed cursor')
-        if self.rowSets is None:
+        if self._rowSets is None:
             raise PyodbcMock.ProgrammingError('Attempt to iterate a cursor before execute')
 
-        if self.ixRowSet >= len(self.rowSets) or self.ixRow >= len(self.rowSets[self.ixRowSet][1]):
+        if self._ixRowSet >= len(self._rowSets) or self._ixRow >= len(self._rowSets[self._ixRowSet][1]):
             raise StopIteration
 
-        rowSet = self.rowSets[self.ixRowSet]
-        row = PyodbcRowMock(rowSet[0], rowSet[1][self.ixRow])
-        self.ixRow += 1
+        rowSet = self._rowSets[self._ixRowSet]
+        row = PyodbcRowMock(rowSet[0], rowSet[1][self._ixRow])
+        self._ixRow += 1
         return row
 
     def next(self):
